"use strict";

import MongoDB from "../../src/Component/DataSource/MongoDB.js";
import { assert } from "chai";
let mongoDB = null;
const collection = "user";
let documentId = null;

before(async function () {
    this.timeout(30000);
    mongoDB = new MongoDB({
        dataSource: "MongoDB",
        host: "db.emvtech.com.br",
        port: 27017,
        user: "emv",
        password: "emv@mongodb123%%!",
        database: "vmagic",
        options: {
            useNewUrlParser: true,
            useUnifiedTopology: true,
            maxPoolSize: 30,
        },
    });
    await mongoDB.createPool();
});

describe("MongoDB.", function () {
    it("Should save an object.", async function () {
        const res = await mongoDB.save(collection, {
            username: "Test",
            password: "Test password.",
        });
        assert.isNotEmpty(res.insertedId);
    });

    xit("Should return all documents.", async function () {
        const res = await mongoDB.findAll(collection);
        assert.ok(res.rows.length > 0);
    });

    xit("Should execute saveMany.", async function () {
        const res = await mongoDB.saveMany(collection, [
            { username: "Test 1", password: "Teste pwd 1" },
            { username: "Test 2", password: "Teste pwd 2" },
        ]);
        documentId = res.insertedIds[0];
        assert.ok(res.rows.length > 0);
    });

    xit("Should update one register", async function () {
        const res = await mongoDB.update(collection, { username: "Test 1 updated", password: "Teste pwd 1 updated" }, { _id: documentId });
        assert.equal("Test 1 updated", res.rows[0].username);
        assert.ok(res.rows.length > 0);
    });

    xit("Should delete from the documentId", async function () {
        const res = await mongoDB.delete(collection, { username: "Test 1 updated", password: "Teste pwd 1 updated" }, { _id: documentId });
        assert.ok(res.deletedCount > 0);
    });
});
