/*jslint devel: true, node: true, indent: 4, unparam: true*/
"use strict";

import Magic from "../src/Magic.js";
import FactoryController from "../src/Controller/FactoryController.js";
import AppDatasource from "../src/Component/DataSource/AppDatasource.js";
let header = null;
let formData = null;

class MagicTest {
    constructor() {
        this.magic = new Magic();
        this.factoryController = new FactoryController(this.magic.getApplication(), "test");
        this.init();
    }

    async init() {
        await this.factoryController.init();
        new AppDatasource(this.factoryController.core.dataSources);
    }

    setHeader(data) {
        header = data;
    }

    setFormData(data) {
        formData = data;
    }

    async controllerTest(name) {
        const instance = await this.factoryController.controller(name);
        instance.headersInfo = header;
        const { dataSource } = this.factoryController.core.dataSources.test;

        if (dataSource === "MongoDB") {
            await (() => {
                return new Promise(resolve => {
                    setTimeout(resolve, 5000);
                });
            })();
        }

        if (typeof instance.beforeFilter !== "undefined") {
            await instance.beforeFilter();
        }

        if (typeof instance.init !== "init") {
            await instance.init();
        }

        instance.getFormData = function () {
            if (!formData) {
                throw new Error("Form data is null.");
            }

            return formData;
        };

        return instance;
    }

    modelTest(name) {
        return this.factoryController.model(name);
    }
}

export default MagicTest;
